<?php

namespace EventPortal\Domain\Delegate;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Enums\InertiaMethod;
use Illuminate\Support\Collection;

class DelegateDashboard
{
    /**
     * @var Delegate
     */
    private $delegate;

    public function __construct(Delegate $delegate)
    {
        $this->delegate = $delegate;
    }

    public static function make(Delegate $delegate): self
    {
        return new self($delegate);
    }

    public function get(): array
    {
        $upcoming = Event::forDelegate($this->delegate)
            ->viewable()
            ->upcoming()
            ->get();

        $previous = Event::forDelegate($this->delegate)
            ->viewable()
            ->previous()
            ->take(config('dashboards.delegate.show_max_previous', 5))
            ->get();

        $upcoming->transform(function (Event $event): array {
            return $this->getFormattedData($event);
        });

        $previous->transform(function (Event $event): array {
            return $this->getFormattedData($event);
        });

        $data = array_filter([
            $this->formatEvents($upcoming, 'Your upcoming events'),
            $this->formatEvents($previous, 'Your previous events'),
        ]);

        return array_values($data);
    }

    private function formatEvents(Collection $events, string $title): array
    {
        if ($events->isEmpty()) {
            return [];
        }

        return [
            'title' => $title,
            'rows' => $events->values()->toArray(),
        ];
    }

    private function getFormattedData(Event $event): array
    {
        $eventDelegate = EventDelegate::forEventAndDelegate($event, $this->delegate)->first();

        return [
            'data' => [
                $event->name,
                $event->readableStartDate,
                $eventDelegate->readableStatus,
            ],
            'buttons' => [
                Button::make('View event')
                    ->method(InertiaMethod::VISIT())
                    ->route('delegate.landing', ['event' => $event->slug])
                    ->icon('chevron-right', 'right')
                    ->variant('secondary'),
            ],
        ];
    }
}
